@@ -28,7 +28,8 @@ def install_demo(home, backend):
         class config:
             POSTGRESQL_DATABASE = {'database': 'rounduptest'}
             DATABASE = 'home'
-        module.db_nuke(config, 1)
+        if module.db_exists(config):
+            module.db_nuke(config)
 
     init.install(home, os.path.join('templates', 'classic'))
     # don't have email flying around
