@@ -75,7 +75,7 @@ class FormItem:
         self.value = value
         if hasattr(self.value, 'filename'):
             self.filename = self.value.filename
-            self.file = self.value
+            self.value = self.value.read()
 
 class FormWrapper:
     '''Make a Zope form dict look like a cgi.py one
@@ -83,11 +83,21 @@ class FormWrapper:
     def __init__(self, form):
         self.form = form
     def __getitem__(self, item):
-        return FormItem(self.form[item])
+        entry = self.form[item]
+        if isinstance(entry, type([])):
+            entry = map(FormItem, entry)
+        else:
+            entry = FormItem(entry)
+        return entry
+    def getvalue(self, key, default=None):
+        if self.form.has_key(key):
+            return self.form[key]
+        else:
+            return default
     def has_key(self, item):
         return self.form.has_key(item)
     def keys(self):
-        return self.form.keys()
+        return self.form.keys():
 
 class ZRoundup(Item, PropertyManager, Implicit, Persistent):
     '''An instance of this class provides an interface between Zope and
