@@ -617,10 +617,13 @@ Command help:
             if ',' in value:
                 values = value.split(',')
             else:
-                values = []
+                values = [value]
             d = props[propname] = {}
             for value in values:
-                d[hyperdb.rawToHyperdb(self.db, cl, None, propname, value)] = 1
+                value = hyperdb.rawToHyperdb(self.db, cl, None, propname, value)
+                if isinstance(value, type([])):
+                    value = value[0]
+                d[value] = 1
 
         # now do the find 
         try:
@@ -1091,7 +1094,7 @@ Command help:
             cl = self.get_class(classname)
 
             # ensure that the properties and the CSV file headings match
-            f = open(os.path.join(dir, file))
+            f = open(os.path.join(dir, file), 'rU')
             reader = rcsv.reader(f, rcsv.colon_separated)
             file_props = None
             maxid = 1
@@ -1109,7 +1112,7 @@ Command help:
             f.close()
 
             # import the journals
-            f = open(os.path.join(args[0], classname + '-journals.csv'))
+            f = open(os.path.join(args[0], classname + '-journals.csv'), 'rU')
             reader = rcsv.reader(f, rcsv.colon_separated)
             cl.import_journals(reader)
             f.close()
