@@ -36,7 +36,10 @@ for backend in ['anydbm', ('mysql', 'MySQLdb'), ('bsddb', '_bsddb'),
         globals()[backend] = __import__('back_%s'%backend, globals())
         __all__.append(backend)
     except ImportError, e:
-        if not str(e).startswith('No module named %s'%backend_module):
-            raise
+        if str(e).startswith('No module named %s'%backend_module):
+            continue
+        if str(e).startswith('No module named %s'%backend):
+            continue
+        raise
 
 # vim: set filetype=python ts=4 sw=4 et si
