@@ -41,7 +41,6 @@ from indexer_dbm import Indexer
 from roundup.backends import locking
 from roundup.hyperdb import String, Password, Date, Interval, Link, \
     Multilink, DatabaseError, Boolean, Number, Node
-from roundup.date import Range
 
 #
 # Now the database
@@ -385,12 +384,14 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
         properties = self.getclass(classname).getprops()
         d = {}
         for k, v in node.items():
-            # if the property doesn't exist, or is the "retired" flag then
-            # it won't be in the properties dict
-            if not properties.has_key(k):
+            if k == self.RETIRED_FLAG:
                 d[k] = v
                 continue
 
+            # if the property doesn't exist then we really don't care
+            if not properties.has_key(k):
+                continue
+
             # get the property spec
             prop = properties[k]
 
@@ -708,6 +709,7 @@ class Database(FileStorage, hyperdb.Database, roundupdb.Database):
             if isinstance(params, type({})):
                 if action in ('set', 'create'):
                     params = self.serialise(classname, params)
+            journaldate = journaldate.serialise()
             l.append((nodeid, journaldate, journaltag, action, params))
         db = self.getCachedJournalDB(classname)
         db[nodeid] = marshal.dumps(l)
@@ -908,7 +910,9 @@ class Class(hyperdb.Class):
             elif isinstance(prop, String):
                 if type(value) != type('') and type(value) != type(u''):
                     raise TypeError, 'new property "%s" not a string'%key
-                self.db.indexer.add_text((self.classname, newid, key), value)
+                if prop.indexme:
+                    self.db.indexer.add_text((self.classname, newid, key),
+                        value)
 
             elif isinstance(prop, Password):
                 if not isinstance(value, password.Password):
@@ -942,8 +946,6 @@ class Class(hyperdb.Class):
                 raise ValueError, 'key property "%s" is required'%key
             if isinstance(prop, Multilink):
                 propvalues[key] = []
-            else:
-                propvalues[key] = None
 
         # done
         self.db.addnode(self.classname, newid, propvalues)
@@ -1075,6 +1077,13 @@ class Class(hyperdb.Class):
         '''
         self.fireAuditors('set', nodeid, propvalues)
         oldvalues = copy.deepcopy(self.db.getnode(self.classname, nodeid))
+        for name,prop in self.getprops(protected=0).items():
+            if oldvalues.has_key(name):
+                continue
+            if isinstance(prop, Multilink):
+                oldvalues[name] = []
+            else:
+                oldvalues[name] = None
         propvalues = self.set_inner(nodeid, **propvalues)
         self.fireReactors('set', nodeid, oldvalues)
         return propvalues        
@@ -1224,8 +1233,9 @@ class Class(hyperdb.Class):
             elif isinstance(prop, String):
                 if value is not None and type(value) != type('') and type(value) != type(u''):
                     raise TypeError, 'new property "%s" not a string'%propname
-                self.db.indexer.add_text((self.classname, nodeid, propname),
-                    value)
+                if prop.indexme:
+                    self.db.indexer.add_text((self.classname, nodeid, propname),
+                        value)
 
             elif isinstance(prop, Password):
                 if not isinstance(value, password.Password):
@@ -1431,6 +1441,8 @@ class Class(hyperdb.Class):
                 node = self.db.getnode(self.classname, nodeid, cldb)
                 if node.has_key(self.db.RETIRED_FLAG):
                     continue
+                if not node.has_key(self.key):
+                    continue
                 if node[self.key] == keyvalue:
                     return nodeid
         finally:
@@ -1587,9 +1599,7 @@ class Class(hyperdb.Class):
 
         The filter must match all properties specificed - but if the
         property value to match is a list, any one of the values in the
-        list may match for that property to match. Unless the property
-        is a Multilink, in which case the item's property list must
-        match the filterspec list.
+        list may match for that property to match.
         """
         if __debug__:
             start_t = time.time()
@@ -1640,14 +1650,14 @@ class Class(hyperdb.Class):
                 l.append((STRING, k, m))
             elif isinstance(propclass, Date):
                 try:
-                    date_rng = Range(v, date.Date, offset=timezone)
+                    date_rng = date.Range(v, date.Date, offset=timezone)
                     l.append((DATE, k, date_rng))
                 except ValueError:
                     # If range creation fails - ignore that search parameter
                     pass
             elif isinstance(propclass, Interval):
                 try:
-                    intv_rng = Range(v, date.Interval)
+                    intv_rng = date.Range(v, date.Interval)
                     l.append((INTERVAL, k, intv_rng))
                 except ValueError:
                     # If range creation fails - ignore that search parameter
@@ -1697,7 +1707,7 @@ class Class(hyperdb.Class):
                     # get the node value
                     nv = node.get(k, None)
 
-                    match = False
+                    match = 0
 
                     # now apply the property filter
                     if t == LINK:
@@ -1717,10 +1727,9 @@ class Class(hyperdb.Class):
                             # othewise, make sure this node has each of the
                             # required values
                             for want in v:
-                                if want not in nv:
+                                if want in nv:
+                                    match = 1
                                     break
-                            else:
-                                match = True
                     elif t == STRING:
                         if nv is None:
                             nv = ''
@@ -1732,10 +1741,10 @@ class Class(hyperdb.Class):
                         else:
                             if v.to_value:
                                 if v.from_value <= nv and v.to_value >= nv:
-                                    match = True
+                                    match = 1
                             else:
                                 if v.from_value <= nv:
-                                    match = True
+                                    match = 1
                     elif t == OTHER:
                         # straight value comparison for the other types
                         match = nv in v
@@ -2034,18 +2043,25 @@ class Class(hyperdb.Class):
             for nodeid, date, user, action, params in self.history(nodeid):
                 date = date.get_tuple()
                 if action == 'set':
+                    export_data = {}
                     for propname, value in params.items():
+                        if not properties.has_key(propname):
+                            # property no longer in the schema
+                            continue
+
                         prop = properties[propname]
                         # make sure the params are eval()'able
                         if value is None:
-                            pass
+                            # don't export empties
+                            continue
                         elif isinstance(prop, Date):
                             value = value.get_tuple()
                         elif isinstance(prop, Interval):
                             value = value.get_tuple()
                         elif isinstance(prop, Password):
                             value = str(value)
-                        params[propname] = value
+                        export_data[propname] = value
+                    params = export_data
                 l = [nodeid, date, user, action, params]
                 r.append(map(repr, l))
         return r
@@ -2066,6 +2082,9 @@ class Class(hyperdb.Class):
                     if value is None:
                         pass
                     elif isinstance(prop, Date):
+                        if type(value) == type(()):
+                            print 'warning: invalid date tuple %r'%(value,)
+                            value = date.Date("2000-1-1")
                         value = date.Date(value)
                     elif isinstance(prop, Interval):
                         value = date.Interval(value)
@@ -2079,7 +2098,7 @@ class Class(hyperdb.Class):
         for nodeid, l in d.items():
             self.db.setjournal(self.classname, nodeid, l)
 
-class FileClass(Class, hyperdb.FileClass):
+class FileClass(hyperdb.FileClass, Class):
     '''This class defines a large chunk of data. To support this, it has a
        mandatory String property "content" which is typically saved off
        externally to the hyperdb.
@@ -2107,10 +2126,6 @@ class FileClass(Class, hyperdb.FileClass):
         # do the database create
         newid = self.create_inner(**propvalues)
 
-        # and index!
-        self.db.indexer.add_text((self.classname, newid, 'content'), content,
-            mime_type)
-
         # fire reactors
         self.fireReactors('create', newid, None)
 
@@ -2140,7 +2155,16 @@ class FileClass(Class, hyperdb.FileClass):
         ''' Snarf the "content" propvalue and update it in a file
         '''
         self.fireAuditors('set', itemid, propvalues)
+
+        # create the oldvalues dict - fill in any missing values
         oldvalues = copy.deepcopy(self.db.getnode(self.classname, itemid))
+        for name,prop in self.getprops(protected=0).items():
+            if oldvalues.has_key(name):
+                continue
+            if isinstance(prop, Multilink):
+                oldvalues[name] = []
+            else:
+                oldvalues[name] = None
 
         # now remove the content property so it's not stored in the db
         content = None
@@ -2166,15 +2190,30 @@ class FileClass(Class, hyperdb.FileClass):
         return propvalues
 
     def getprops(self, protected=1):
-        ''' In addition to the actual properties on the node, these methods
-            provide the "content" property. If the "protected" flag is true,
-            we include protected properties - those which may not be
-            modified.
+        '''In addition to the actual properties on the node, these methods
+        provide the "content" property. If the "protected" flag is true,
+        we include protected properties - those which may not be
+        modified.
+
+        Note that the content prop is indexed separately, hence no indexme.
         '''
         d = Class.getprops(self, protected=protected).copy()
         d['content'] = hyperdb.String()
         return d
 
+    def index(self, nodeid):
+        '''Add (or refresh) the node to search indexes.
+
+        Pass on the content-type property for the content property.
+        '''
+        Class.index(self, nodeid)
+        try:
+            mime_type = self.get(nodeid, 'type', self.default_mime_type)
+        except KeyError:
+            mime_type = self.default_mime_type
+        self.db.indexer.add_text((self.classname, nodeid, 'content'),
+            str(self.get(nodeid, 'content')), mime_type)
+
 # deviation from spec - was called ItemClass
 class IssueClass(Class, roundupdb.IssueClass):
     # Overridden methods:
