@@ -105,6 +105,9 @@ def db_exists(config):
 class Database(Database):
     arg = '%s'
 
+    # used by some code to switch styles of query
+    implements_intersect = 0
+
     # Backend for MySQL to use.
     # InnoDB is faster, but if you're running <4.0.16 then you'll need to
     # use BDB to pass all unit tests.
@@ -170,14 +173,14 @@ class Database(Database):
     def create_version_2_tables(self):
         # OTK store
         self.cursor.execute('''CREATE TABLE otks (otk_key VARCHAR(255),
-            otk_value VARCHAR(255), otk_time FLOAT(20))
+            otk_value TEXT, otk_time FLOAT(20))
             TYPE=%s'''%self.mysql_backend)
         self.cursor.execute('CREATE INDEX otks_key_idx ON otks(otk_key)')
 
         # Sessions store
         self.cursor.execute('''CREATE TABLE sessions (
             session_key VARCHAR(255), session_time FLOAT(20),
-            session_value VARCHAR(255)) TYPE=%s'''%self.mysql_backend)
+            session_value TEXT) TYPE=%s'''%self.mysql_backend)
         self.cursor.execute('''CREATE INDEX sessions_key_idx ON
             sessions(session_key)''')
 
@@ -252,7 +255,7 @@ class Database(Database):
             olddata = []
             propnames = propnames + ['id', '__retired__']
             cols = []
-            first = True
+            first = 1
             for entry in self.cursor.fetchall():
                 l = []
                 olddata.append(l)
@@ -292,7 +295,7 @@ class Database(Database):
                             l.append(v.as_seconds())
                         else:
                             l.append(e)
-                first = False
+                first = 0
 
             self.drop_class_table_indexes(cn, old_spec[0])
 
@@ -466,7 +469,7 @@ class Database(Database):
         ''' Generate a new id for the given class
         '''
         # get the next ID
-        sql = 'select num from ids where name=%s'%self.arg
+        sql = 'select num from ids where name=%s FOR UPDATE'%self.arg
         if __debug__:
             print >>hyperdb.DEBUG, 'newid', (self, sql, classname)
         self.cursor.execute(sql, (classname, ))
@@ -535,7 +538,7 @@ class MysqlClass:
         property value to match is a list, any one of the values in the
         list may match for that property to match.
         '''
-        # just don't bother if the full-text search matched diddly
+        # we can't match anything if search_matches is empty
         if search_matches == {}:
             return []
 
@@ -602,25 +605,30 @@ class MysqlClass:
                 v = ['%%'+self.db.sql_stringquote(s)+'%%' for s in v]
 
                 # now add to the where clause
-                where.append(' or '.join(["_%s._%s LIKE '%s'"%(cn, k, s)
-                    for s in v]))
+                where.append('('
+                    +' or '.join(["_%s._%s LIKE '%s'"%(cn, k, s) for s in v])
+                    +')')
                 # note: args are embedded in the query string now
             elif isinstance(propclass, Link):
                 if isinstance(v, type([])):
-                    if '-1' in v:
-                        v = v[:]
-                        v.remove('-1')
-                        xtra = ' or _%s._%s is NULL'%(cn, k)
-                    else:
-                        xtra = ''
-                    if v:
+                    d = {}
+                    for entry in v:
+                        if entry == '-1':
+                            entry = None
+                        d[entry] = entry
+                    l = []
+                    if d.has_key(None) or not d:
+                        del d[None]
+                        l.append('_%s._%s is NULL'%(cn, k))
+                    if d:
+                        v = d.keys()
                         s = ','.join([a for x in v])
-                        where.append('(_%s._%s in (%s)%s)'%(cn, k, s, xtra))
+                        l.append('(_%s._%s in (%s))'%(cn, k, s))
                         args = args + v
-                    else:
-                        where.append('_%s._%s is NULL'%(cn, k))
+                    if l:
+                        where.append('(' + ' or '.join(l) +')')
                 else:
-                    if v == '-1':
+                    if v in ('-1', None):
                         v = None
                         where.append('_%s._%s is NULL'%(cn, k))
                     else:
@@ -683,6 +691,10 @@ class MysqlClass:
             where.append('_%s.id in (%s)'%(cn, s))
             args = args + v
 
+        # sanity check: sorting *and* grouping on the same property?
+        if group[1] == sort[1]:
+            sort = (None, None)
+
         # "grouping" is just the first-order sorting in the SQL fetch
         orderby = []
         ordercols = []
