@@ -89,6 +89,9 @@ def db_exists(config):
 class Database(rdbms_common.Database):
     arg = '%s'
 
+    # used by some code to switch styles of query
+    implements_intersect = 1
+
     def sql_open_connection(self):
         db = getattr(self.config, 'POSTGRESQL_DATABASE')
         try:
@@ -122,13 +125,13 @@ class Database(rdbms_common.Database):
     def create_version_2_tables(self):
         # OTK store
         self.cursor.execute('''CREATE TABLE otks (otk_key VARCHAR(255),
-            otk_value VARCHAR(255), otk_time REAL)''')
+            otk_value TEXT, otk_time REAL)''')
         self.cursor.execute('CREATE INDEX otks_key_idx ON otks(otk_key)')
 
         # Sessions store
         self.cursor.execute('''CREATE TABLE sessions (
             session_key VARCHAR(255), session_time REAL,
-            session_value VARCHAR(255))''')
+            session_value TEXT)''')
         self.cursor.execute('''CREATE INDEX sessions_key_idx ON
             sessions(session_key)''')
 
@@ -159,6 +162,11 @@ class Database(rdbms_common.Database):
             c.execute('CREATE INDEX %ss_key_idx ON %ss(%s_key)'%(name, name,
                 name))
 
+    def fix_version_3_tables(self):
+        rdbms_common.Database.fix_version_3_tables(self)
+        self.cursor.execute('''CREATE INDEX words_both_idx ON public.__words
+            USING btree (_word, _textid)''')
+
     def add_actor_column(self):
         # update existing tables to have the new actor column
         tables = self.database_schema['tables']
