@@ -19,6 +19,10 @@ import sqlite
 class Database(rdbms_common.Database):
     # char to use for positional arguments
     arg = '%s'
+
+    # used by some code to switch styles of query
+    implements_intersect = 1
+
     hyperdb_to_sql_datatypes = {
         hyperdb.String : 'VARCHAR(255)',
         hyperdb.Date   : 'VARCHAR(30)',
@@ -106,6 +110,10 @@ class Database(rdbms_common.Database):
                 # we've updated - don't try again
                 tables[classname] = spec.schema()
 
+    def fix_version_3_tables(self):
+        # NOOP - no restriction on column length here
+        pass
+
     def update_class(self, spec, old_spec, force=0, adding_v2=0):
         ''' Determine the differences between the current spec and the
             database version of the spec, and update where necessary.
