@@ -127,6 +127,8 @@ class SearchAction(Action):
 
         # handle saving the query params
         if queryname:
+            if not self.hasPermission('Edit', 'query'):
+                raise Unauthorised, _("You do not have permission to edit queries")
             # parse the environment and figure what the query _is_
             req = templating.HTMLRequest(self.client)
 
@@ -667,7 +669,7 @@ class ConfRegoAction(Action):
             return
 
         # log the new user in
-        self.client.user = self.db.user.get(self.userid, 'username')
+        self.user = self.client.user = self.db.user.get(self.userid, 'username')
         # re-open the database for real, using the user
         self.client.opendb(self.client.user)
 
@@ -801,14 +803,14 @@ class LoginAction(Action):
             self.client.userid = self.db.user.lookup(self.client.user)
         except KeyError:
             name = self.client.user
-            self.client.error_message.append(_('No such user "%(name)s"')%locals())
+            self.client.error_message.append(_('Invalid login'))
             self.client.make_user_anonymous()
             return
 
         # verify the password
         if not self.verifyPassword(self.client.userid, password):
             self.client.make_user_anonymous()
-            self.client.error_message.append(_('Incorrect password'))
+            self.client.error_message.append(_('Invalid login'))
             return
 
         # Determine whether the user has permission to log in.
@@ -859,7 +861,13 @@ class ExportCSVAction(Action):
         h['Content-Type'] = 'text/csv'
         # some browsers will honor the filename here...
         h['Content-Disposition'] = 'inline; filename=query.csv'
+
         self.client.header()
+
+        if self.client.env['REQUEST_METHOD'] == 'HEAD':
+            # all done, return a dummy string
+            return 'dummy'
+
         writer = rcsv.writer(self.client.request.wfile)
         writer.writerow(columns)
 
