@@ -187,15 +187,12 @@ class Client:
         self.ok_message = []
         self.error_message = []
         try:
-            # figure out the context and desired content template
-            # do this first so we don't authenticate for static files
-            # Note: this method opens the database as "admin" in order to
-            # perform context checks
-            self.determine_context()
-
             # make sure we're identified (even anonymously)
             self.determine_user()
 
+            # figure out the context and desired content template
+            self.determine_context()
+
             # possibly handle a form submit action (may change self.classname
             # and self.template, and may also append error/ok_messages)
             html = self.handle_action()
@@ -211,7 +208,9 @@ class Client:
 #            self.additional_headers['Pragma'] = 'no-cache'
 
             # expire this page 5 seconds from now
-            date = rfc822.formatdate(time.time() + 5)
+            # <rj> changed to "fix" IE caching issue - always expire all
+            # pages
+            date = rfc822.formatdate(time.time() - 1)
             self.additional_headers['Expires'] = date
 
             # render the content
@@ -284,14 +283,12 @@ class Client:
         self.clean_sessions()
         sessions = self.db.getSessionManager()
 
-        user = 'anonymous'
-
         # first up, try the REMOTE_USER var (from HTTP Basic Auth handled
         # by a front-end HTTP server)
-        try:
-            user = os.getenv('REMOTE_USER')
-        except KeyError:
-            pass
+        if self.env.has_key('REMOTE_USER'):
+            user = self.env['REMOTE_USER']
+        else:
+            user = 'anonymous'
 
         # look up the user session cookie (may override the REMOTE_USER)
         cookie = Cookie.SimpleCookie(self.env.get('HTTP_COOKIE', ''))
@@ -403,9 +400,6 @@ class Client:
                 # send the file identified by the designator in path[0]
                 raise SendFile, path[0]
 
-        # we need the db for further context stuff - open it as admin
-        self.opendb('admin')
-
         # see if we got a designator
         m = dre.match(self.classname)
         if m:
@@ -437,7 +431,6 @@ class Client:
             raise NotFound, str(designator)
         classname, nodeid = m.group(1), m.group(2)
 
-        self.opendb('admin')
         klass = self.db.getclass(classname)
 
         # make sure we have the appropriate properties
@@ -447,6 +440,11 @@ class Client:
         if not props.has_key('content'):
             raise NotFound, designator
 
+        # make sure we have permission
+        if not self.db.security.hasPermission('View', self.userid, classname):
+            raise Unauthorised, self._("You are not allowed to view "
+                "this file.")
+
         mime_type = klass.get(nodeid, 'type')
         content = klass.get(nodeid, 'content')
         lmt = klass.get(nodeid, 'activity').timestamp()
@@ -460,6 +458,9 @@ class Client:
         # outside of the static files dir
         prefix = getattr(self.instance.config, 'STATIC_FILES',
             self.instance.config.TEMPLATES)
+
+        # normalise the prefix and filename for the startswith comparison
+        prefix = os.path.normpath(prefix)
         filename = os.path.normpath(os.path.join(prefix, file))
         if not filename.startswith(prefix):
             raise NotFound, file
@@ -505,7 +506,7 @@ class Client:
         self.additional_headers['Content-Type'] = mime_type
         self.additional_headers['Content-Length'] = len(content)
         lmt = rfc822.formatdate(lmt)
-        self.additional_headers['Last-Modifed'] = lmt
+        self.additional_headers['Last-Modified'] = lmt
         self.write(content)
 
     def renderContext(self):
@@ -596,7 +597,8 @@ class Client:
     def write(self, content):
         if not self.headers_done:
             self.header()
-        self.request.wfile.write(content)
+        if self.env['REQUEST_METHOD'] != 'HEAD':
+            self.request.wfile.write(content)
 
     def setHeader(self, header, value):
         '''Override a header to be returned to the user's browser.
