@@ -2,8 +2,8 @@
 """
 
 todo = '''
-- Most methods should have a "default" arg to supply a value 
-  when none appears in the hyperdb or request. 
+- Most methods should have a "default" arg to supply a value
+  when none appears in the hyperdb or request.
 - Multilink property additions: change_note and new_upload
 - Add class.find() too
 - NumberHTMLProperty should support numeric operations
@@ -48,12 +48,12 @@ class Unauthorised(Exception):
         return 'You are not allowed to %s items of class %s'%(self.action,
             self.klass)
 
-def find_template(dir, name, extension):
+def find_template(dir, name, view):
     ''' Find a template in the nominated dir
     '''
     # find the source
-    if extension:
-        filename = '%s.%s'%(name, extension)
+    if view:
+        filename = '%s.%s'%(name, view)
     else:
         filename = name
 
@@ -63,17 +63,18 @@ def find_template(dir, name, extension):
         return (src, filename)
 
     # try with a .html extension (new-style)
-    filename = filename + '.html'
-    src = os.path.join(dir, filename)
-    if os.path.exists(src):
-        return (src, filename)
+    for extension in '.html', '.xml':
+        filename = filename + extension
+        src = os.path.join(dir, filename)
+        if os.path.exists(src):
+            return (src, filename)
 
-    # no extension == no generic template is possible
-    if not extension:
+    # no view == no generic template is possible
+    if not view:
         raise NoTemplate, 'Template file "%s" doesn\'t exist'%name
 
     # try for a _generic template
-    generic = '_generic.%s'%extension
+    generic = '_generic.%s'%view
     src = os.path.join(dir, generic)
     if os.path.exists(src):
         return (src, generic)
@@ -85,7 +86,7 @@ def find_template(dir, name, extension):
         return (src, generic)
 
     raise NoTemplate, 'No template file exists for templating "%s" '\
-        'with template "%s" (neither "%s" nor "%s")'%(name, extension,
+        'with template "%s" (neither "%s" nor "%s")'%(name, view,
         filename, generic)
 
 class Templates:
@@ -98,7 +99,18 @@ class Templates:
         ''' Go through a directory and precompile all the templates therein
         '''
         for filename in os.listdir(self.dir):
-            if os.path.isdir(filename): continue
+            # skip subdirs
+            if os.path.isdir(filename):
+                continue
+
+            # skip files without ".html" or ".xml" extension - .css, .js etc.
+            for extension in '.html', '.xml':
+                if filename.endswith(extension):
+                    break
+            else:
+                continue
+
+            # load the template
             if '.' in filename:
                 name, extension = filename.split('.')
                 self.get(name, extension)
@@ -174,7 +186,7 @@ class RoundupPageTemplate(PageTemplate.PageTemplate):
 
        - the url
        - the current index information (``filterspec``, ``filter`` args,
-         ``properties``, etc) parsed out of the form. 
+         ``properties``, etc) parsed out of the form.
        - methods for easy filterspec link generation
        - *user*, the current user node as an HTMLItem instance
        - *form*, the current CGI form information as a FieldStorage
@@ -550,7 +562,7 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
             height='400', property=''):
         ''' Pop up a javascript window with class help
 
-            This generates a link to a popup window which displays the 
+            This generates a link to a popup window which displays the
             properties indicated by "properties" of the class named by
             "classname". The "properties" should be a comma-separated list
             (eg. 'id,name,description'). Properties defaults to all the
@@ -665,7 +677,7 @@ class HTMLItem(HTMLInputMixin, HTMLPermissions):
     def is_retired(self):
         """Is this item retired?"""
         return self._klass.is_retired(self._nodeid)
-    
+
     def submit(self, label="Submit Changes"):
         """Generate a submit button.
 
@@ -685,44 +697,37 @@ class HTMLItem(HTMLInputMixin, HTMLPermissions):
     def history(self, direction='descending', dre=re.compile('^\d+$')):
         self.view_check()
 
-        l = ['<table class="history">'
-             '<tr><th colspan="4" class="header">',
-             _('History'),
-             '</th></tr><tr>',
-             _('<th>Date</th>'),
-             _('<th>User</th>'),
-             _('<th>Action</th>'),
-             _('<th>Args</th>'),
-            '</tr>']
+        l = []
         current = {}
         comments = {}
         history = self._klass.history(self._nodeid)
         history.sort()
         timezone = self._db.getUserTimezone()
-        if direction == 'descending':
-            history.reverse()
-            # pre-load the history with the current state
-            for prop_n in self._props.keys():
-                prop = self[prop_n]
-                if not isinstance(prop, HTMLProperty):
-                    continue
-                current[prop_n] = prop.plain()
-                # make link if hrefable
-                if (self._props.has_key(prop_n) and
-                        isinstance(self._props[prop_n], hyperdb.Link)):
-                    classname = self._props[prop_n].classname
-                    try:
-                        template = find_template(self._db.config.TEMPLATES,
-                            classname, 'item')
-                        if template[1].startswith('_generic'):
-                            raise NoTemplate, 'not really...'
-                    except NoTemplate:
-                        pass
-                    else:
-                        id = self._klass.get(self._nodeid, prop_n, None)
-                        current[prop_n] = '<a href="%s%s">%s</a>'%(
-                            classname, id, current[prop_n])
- 
+
+        history.reverse()
+
+        # pre-load the history with the current state
+        for prop_n in self._props.keys():
+            prop = self[prop_n]
+            if not isinstance(prop, HTMLProperty):
+                continue
+            current[prop_n] = prop.plain()
+            # make link if hrefable
+            if (self._props.has_key(prop_n) and
+                    isinstance(self._props[prop_n], hyperdb.Link)):
+                classname = self._props[prop_n].classname
+                try:
+                    template = find_template(self._db.config.TEMPLATES,
+                        classname, 'item')
+                    if template[1].startswith('_generic'):
+                        raise NoTemplate, 'not really...'
+                except NoTemplate:
+                    pass
+                else:
+                    id = self._klass.get(self._nodeid, prop_n, None)
+                    current[prop_n] = '<a href="%s%s">%s</a>'%(
+                        classname, id, current[prop_n])
+
         for id, evt_date, user, action, args in history:
             date_s = str(evt_date.local(timezone)).replace("."," ")
             arg_s = ''
@@ -893,6 +898,19 @@ class HTMLItem(HTMLInputMixin, HTMLPermissions):
             l.append(_('<tr><td colspan=4><strong>Note:</strong></td></tr>'))
         for entry in comments.values():
             l.append('<tr><td colspan=4>%s</td></tr>'%entry)
+
+        if direction == 'ascending':
+            l.reverse()
+
+        l[0:0] = ['<table class="history">'
+             '<tr><th colspan="4" class="header">',
+             _('History'),
+             '</th></tr><tr>',
+             _('<th>Date</th>'),
+             _('<th>User</th>'),
+             _('<th>Action</th>'),
+             _('<th>Args</th>'),
+            '</tr>']
         l.append('</table>')
         return '\n'.join(l)
 
@@ -950,6 +968,7 @@ class HTMLUserPermission:
         # may anonymous users register? (so, they need to be anonymous,
         # need the Web Rego permission, and not trying to view an item)
         rego = s.hasPermission('Web Registration', userid, self._classname)
+        rego = rego and self._client.template == 'register'
         if is_anonymous and rego and getattr(self, '_nodeid', None) is None:
             return 1
 
@@ -1044,7 +1063,7 @@ class HTMLProperty(HTMLInputMixin, HTMLPermissions):
 class StringHTMLProperty(HTMLProperty):
     hyper_re = re.compile(r'((?P<url>\w{3,6}://\S+)|'
                           r'(?P<email>[-+=%/\w\.]+@[\w\.\-]+)|'
-                          r'(?P<item>(?P<class>[a-z_]+)(?P<id>\d+)))')
+                          r'(?P<item>(?P<class>[A-Za-z_]+)\s*(?P<id>\d+)))')
     def _hyper_repl(self, match):
         if match.group('url'):
             s = match.group('url')
@@ -1058,12 +1077,12 @@ class StringHTMLProperty(HTMLProperty):
             s2 = match.group('id')
             try:
                 # make sure s1 is a valid tracker classname
-                cl = self._db.getclass(s1)
+                cl = self._db.getclass(s1.lower())
                 if not cl.hasnode(s2):
                     raise KeyError, 'oops'
-                return '<a href="%s">%s%s</a>'%(s, s1, s2)
+                return '<a href="%s%s">%s</a>'%(s1, s2, s)
             except KeyError:
-                return '%s%s'%(s1, s2)
+                return s
 
     def hyperlinked(self):
         ''' Render a "hyperlinked" version of the text '''
@@ -1071,7 +1090,7 @@ class StringHTMLProperty(HTMLProperty):
 
     def plain(self, escape=0, hyperlink=0):
         '''Render a "plain" representation of the property
-            
+
         - "escape" turns on/off HTML quoting
         - "hyperlink" turns on/off in-text hyperlinking of URLs, email
           addresses and designators
@@ -1183,7 +1202,7 @@ class PasswordHTMLProperty(HTMLProperty):
         return self.plain()
 
     def confirm(self, size = 30):
-        ''' Render a second form edit field for the property, used for 
+        ''' Render a second form edit field for the property, used for
             confirmation that the user typed the password correctly. Generates
             a field with name "@confirm@name".
 
@@ -1203,6 +1222,9 @@ class NumberHTMLProperty(HTMLProperty):
         '''
         self.view_check()
 
+        if self._value is None:
+            return ''
+
         return str(self._value)
 
     def field(self, size=30):
@@ -1270,6 +1292,9 @@ class BooleanHTMLProperty(HTMLProperty):
         return s
 
 class DateHTMLProperty(HTMLProperty):
+
+    _marker = []
+
     def __init__(self, client, classname, nodeid, prop, name, value,
             anonymous=0, offset=None):
         HTMLProperty.__init__(self, client, classname, nodeid, prop, name,
@@ -1300,24 +1325,35 @@ class DateHTMLProperty(HTMLProperty):
         return DateHTMLProperty(self._client, self._classname, self._nodeid,
             self._prop, self._formname, date.Date('.'))
 
-    def field(self, size = 30):
-        ''' Render a form edit field for the property
+    def field(self, size=30, format=_marker):
+        '''Render a form edit field for the property
 
-            If not editable, just display the value via plain().
+        If not editable, just display the value via plain().
+
+        The format string is a standard python strftime format string.
         '''
         self.view_check()
+        if not self.is_edit_ok():
+            if format is self._marker:
+                return self.plain()
+            else:
+                return self.pretty(format)
 
         if self._value is None:
             value = ''
+        elif type(self._value) is type(''):
+            if format is self._marker:
+                value = self._value
+            else:
+                value = date.Date(self._value).pretty(format)
         else:
             tz = self._db.getUserTimezone()
-            value = cgi.escape(str(self._value.local(tz)))
+            value = self._value.local(tz)
+            if format is not self._marker:
+                value = value.pretty(format)
 
-        if self.is_edit_ok():
-            value = '&quot;'.join(value.split('"'))
-            return self.input(name=self._formname,value=value,size=size)
-        
-        return self.plain()
+        value = cgi.escape(str(value), True)
+        return self.input(name=self._formname, value=value, size=size)
 
     def reldate(self, pretty=1):
         ''' Render the interval between the date and now.
@@ -1335,7 +1371,6 @@ class DateHTMLProperty(HTMLProperty):
             return interval.pretty()
         return str(interval)
 
-    _marker = []
     def pretty(self, format=_marker):
         ''' Render the date in a pretty format (eg. month names, spaces).
 
@@ -1346,7 +1381,9 @@ class DateHTMLProperty(HTMLProperty):
         '''
         self.view_check()
 
-        if format is not self._marker:
+        if not self._value:
+            return ''
+        elif format is not self._marker:
             return self._value.pretty(format)
         else:
             return self._value.pretty()
@@ -1482,9 +1519,9 @@ class LinkHTMLProperty(HTMLProperty):
         if value is None:
             s = 'selected="selected" '
         l.append(_('<option %svalue="-1">- no selection -</option>')%s)
-        if linkcl.getprops().has_key('order'):  
+        if linkcl.getprops().has_key('order'):
             sort_on = ('+', 'order')
-        else:  
+        else:
             if sort_on is None:
                 sort_on = ('+', linkcl.labelprop())
             else:
@@ -1539,7 +1576,7 @@ class MultilinkHTMLProperty(HTMLProperty):
                 fail_ok=1)
             sortfun = make_sort_function(self._db, self._prop.classname)
             self._value.sort(sortfun)
-    
+
     def __len__(self):
         ''' length of the multilink '''
         return len(self._value)
@@ -1718,9 +1755,13 @@ class ShowDict:
     ''' A convenience access to the :columns index parameters
     '''
     def __init__(self, columns):
-        self.columns = {}
-        for col in columns:
-            self.columns[col] = 1
+        if columns:
+            self.columns = {}
+            for col in columns:
+                self.columns[col] = 1
+        else:
+            self.__getitem__ = lambda name: 1
+
     def __getitem__(self, name):
         return self.columns.has_key(name)
 
@@ -1854,7 +1895,7 @@ class HTMLRequest(HTMLInputMixin):
     def updateFromURL(self, url):
         ''' Parse the URL for query args, and update my attributes using the
             values.
-        ''' 
+        '''
         env = {'QUERY_STRING': url}
         self.form = cgi.FieldStorage(environ=env)
 
@@ -1934,9 +1975,13 @@ env: %(env)s
         if filter and self.filter:
             l.append(s%(sc+'filter', ','.join(self.filter)))
         if filterspec:
+            props = self.client.db.getclass(self.classname).getprops()
             for k,v in self.filterspec.items():
                 if type(v) == type([]):
-                    l.append(s%(k, ','.join(v)))
+                    if isinstance(props[k], hyperdb.String):
+                        l.append(s%(k, ' '.join(v)))
+                    else:
+                        l.append(s%(k, ','.join(v)))
                 else:
                     l.append(s%(k, v))
         if self.search_text:
@@ -1982,10 +2027,14 @@ env: %(env)s
             l.append(sc+'startwith=%s'%self.startwith)
 
         # finally, the remainder of the filter args in the request
+        props = self.client.db.getclass(self.classname).getprops()
         for k,v in self.filterspec.items():
             if not args.has_key(k):
                 if type(v) == type([]):
-                    l.append('%s=%s'%(k, ','.join(v)))
+                    if isinstance(props[k], hyperdb.String):
+                        l.append(s%(k, ' '.join(v)))
+                    else:
+                        l.append(s%(k, ','.join(v)))
                 else:
                     l.append('%s=%s'%(k, v))
         return '%s?%s'%(url, '&'.join(l))
@@ -2072,7 +2121,7 @@ class Batch(ZTUtils.Batch):
         if index < 0:
             if index + self.end < self.first: raise IndexError, index
             return self._sequence[index + self.end]
-        
+
         if index >= self.length:
             raise IndexError, index
 
@@ -2134,3 +2183,4 @@ class TemplatingUtils:
         '''HTML-quote the supplied text.'''
         return cgi.escape(url)
 
+# vim: set et sts=4 sw=4 :
