@@ -25,6 +25,12 @@ import time, re, calendar, types
 from types import *
 from i18n import _
 
+try:
+    import datetime
+    have_datetime = 1
+except:
+    have_datetime = 0
+
 def _add_granularity(src, order, value = 1):
     '''Increment first non-None value in src dictionary ordered by 'order'
     parameter
@@ -121,8 +127,15 @@ class Date:
         if type(spec) == type(''):
             self.set(spec, offset=offset, add_granularity=add_granularity)
             return
+        elif have_datetime and isinstance(spec, datetime.datetime):
+            # Python 2.3+ datetime object
+            y,m,d,H,M,S,x,x,x = spec.timetuple()
+            S += spec.microsecond/1000000.
+            spec = (y,m,d,H,M,S,x,x,x)
         elif hasattr(spec, 'tuple'):
             spec = spec.tuple()
+        elif isinstance(spec, Date):
+            spec = spec.get_tuple()
         try:
             y,m,d,H,M,S,x,x,x = spec
             frac = S - int(S)
@@ -348,8 +361,14 @@ class Date:
             Note that if the day is zero, and the day appears first in the
             format, then the day number will be removed from output.
         '''
-        str = time.strftime(format, (self.year, self.month, self.day,
-            self.hour, self.minute, int(self.second), 0, 0, 0))
+        # Python2.4 strftime() enforces the non-zero-ness of the day-of-year
+        # component of the time tuple, so we need to figure it out
+        t = (self.year, self.month, self.day, self.hour, self.minute,
+            int(self.second), 0, 0, 0)
+        t = calendar.timegm(t)
+        t = time.gmtime(t)
+        str = time.strftime(format, t)
+
         # handle zero day by removing it
         if format.startswith('%d') and str[0] == '0':
             return ' ' + str[1:]
@@ -438,6 +457,9 @@ class Interval:
             self.from_seconds(spec)
         elif type(spec) in (StringType, UnicodeType):
             self.set(spec, allowdate=allowdate, add_granularity=add_granularity)
+        elif isinstance(spec, Interval):
+            (self.sign, self.year, self.month, self.day, self.hour,
+                self.minute, self.second) = spec.get_tuple()
         else:
             if len(spec) == 7:
                 self.sign, self.year, self.month, self.day, self.hour, \
