@@ -671,7 +671,7 @@ def rawToHyperdb(db, klass, itemid, propname, value,
     elif isinstance(proptype, Date):
         try:
             tz = db.getUserTimezone()
-            value = date.Date(value).local(tz)
+            value = date.Date(value).local(-tz)
         except ValueError, message:
             raise HyperdbValueError, 'property %s: %r is an invalid '\
                 'date (%s)'%(propname, value, message)
@@ -787,7 +787,7 @@ class FileClass:
     def import_files(self, dirname, nodeid):
         ''' Import the "content" property as a file
         '''
-        dest = self.db.filename(self.classname, nodeid)
+        dest = self.db.filename(self.classname, nodeid, create=1)
         x, filename = os.path.split(dest)
         x, subdir = os.path.split(x)
         source = os.path.join(dirname, self.classname+'-files', subdir,
