@@ -32,11 +32,12 @@ from roundup.i18n import _
 def usage(args, message=None):
     if message is not None:
         print message
-    print _('Usage: %(program)s [-v] [[-C class] -S field=value]* <instance '
-        'home> [method]')%{'program': args[0]}
+    print _('Usage: %(program)s [-v] [-c] [[-C class] -S field=value]* '
+        '<instance home> [method]')%{'program': args[0]}
     print _('''
 Options:
  -v: print version and exit
+ -c: default class of item to create (else the tracker's MAIL_DEFAULT_CLASS)
  -C / -S: see below
 
 The roundup mail gateway may be called in one of four ways:
@@ -103,7 +104,7 @@ def main(argv):
     # take the argv array and parse it leaving the non-option
     # arguments in the args array.
     try:
-        optionsList, args = getopt.getopt(argv[1:], 'vC:S:', ['set=',
+        optionsList, args = getopt.getopt(argv[1:], 'vc:C:S:', ['set=',
             'class='])
     except getopt.GetoptError:
         # print help information and exit:
