@@ -654,8 +654,9 @@ class DBTest(MyTestCase):
         f2 = self.db.file.create(content='world', type="text/frozz",
             comment='blah blah')
         i1 = self.db.issue.create(files=[f1, f2], title="flebble plop")
-        i2 = self.db.issue.create(title="flebble frooz")
+        i2 = self.db.issue.create(title="flebble the frooz")
         self.db.commit()
+        self.assertEquals(self.db.indexer.search([], self.db.issue), {})
         self.assertEquals(self.db.indexer.search(['hello'], self.db.issue),
             {i1: {'files': [f1]}})
         self.assertEquals(self.db.indexer.search(['world'], self.db.issue), {})
@@ -664,6 +665,13 @@ class DBTest(MyTestCase):
         self.assertEquals(self.db.indexer.search(['flebble'], self.db.issue),
             {i1: {}, i2: {}})
 
+        # test AND'ing of search terms
+        self.assertEquals(self.db.indexer.search(['frooz', 'flebble'],
+            self.db.issue), {i2: {}})
+
+        # unindexed stopword
+        self.assertEquals(self.db.indexer.search(['the'], self.db.issue), {})
+
     def testReindexing(self):
         search = self.db.indexer.search
         issue = self.db.issue
@@ -691,6 +699,7 @@ class DBTest(MyTestCase):
         i1 = self.db.issue.create(files=[f1, f2])
         self.db.commit()
         d = self.db.indexer.search(['hello'], self.db.issue)
+        self.assert_(d.has_key(i1))
         d[i1]['files'].sort()
         self.assertEquals(d, {i1: {'files': [f1, f2]}})
         self.assertEquals(self.db.indexer.search(['world'], self.db.issue),
@@ -759,10 +768,12 @@ class DBTest(MyTestCase):
 
     def testFindMultipleLink(self):
         one, two, three, four = self._find_test_setup()
-        self.assertEqual(self.db.issue.find(status={'1':1, '3':1}),
-            [one, three, four])
-        self.assertEqual(self.db.issue.find(assignedto={None:1, '1':1}),
-            [one, three, four])
+        l = self.db.issue.find(status={'1':1, '3':1})
+        l.sort()
+        self.assertEqual(l, [one, three, four])
+        l = self.db.issue.find(assignedto={None:1, '1':1})
+        l.sort()
+        self.assertEqual(l, [one, three, four])
 
     def testFindMultilink(self):
         one, two, three, four = self._find_test_setup()
@@ -835,15 +846,15 @@ class DBTest(MyTestCase):
         iss = self.db.issue
         for issue in (
                 {'title': 'issue one', 'status': '2', 'assignedto': '1',
-                    'foo': date.Interval('1:10'), 'priority': '1',
-                    'deadline': date.Date('2003-01-01.00:00')},
+                    'foo': date.Interval('1:10'), 'priority': '3',
+                    'deadline': date.Date('2003-02-16.22:50')},
                 {'title': 'issue two', 'status': '1', 'assignedto': '2',
                     'foo': date.Interval('1d'), 'priority': '3',
-                    'deadline': date.Date('2003-02-16.22:50')},
-                    {'title': 'issue three', 'status': '1', 'priority': '2',
+                    'deadline': date.Date('2003-01-01.00:00')},
+                {'title': 'issue three', 'status': '1', 'priority': '2',
                     'nosy': ['1','2'], 'deadline': date.Date('2003-02-18')},
                 {'title': 'non four', 'status': '3',
-                    'foo': date.Interval('0:10'), 'priority': '1',
+                    'foo': date.Interval('0:10'), 'priority': '2',
                     'nosy': ['1'], 'deadline': date.Date('2004-03-08')}):
             self.db.issue.create(**issue)
         file_content = ''.join([chr(i) for i in range(255)])
@@ -877,6 +888,11 @@ class DBTest(MyTestCase):
         ae, filt = self.filteringSetup()
         ae(filt(None, {'status': '1'}, ('+','id'), (None,None)), ['2','3'])
         ae(filt(None, {'assignedto': '-1'}, ('+','id'), (None,None)), ['3','4'])
+        ae(filt(None, {'assignedto': None}, ('+','id'), (None,None)), ['3','4'])
+        ae(filt(None, {'assignedto': ['-1', None]}, ('+','id'), (None,None)),
+            ['3','4'])
+        ae(filt(None, {'assignedto': ['1', None]}, ('+','id'), (None,None)),
+            ['1', '3','4'])
 
     def testFilteringRetired(self):
         ae, filt = self.filteringSetup()
@@ -887,6 +903,8 @@ class DBTest(MyTestCase):
         ae, filt = self.filteringSetup()
         ae(filt(None, {'nosy': '2'}, ('+','id'), (None,None)), ['3'])
         ae(filt(None, {'nosy': '-1'}, ('+','id'), (None,None)), ['1', '2'])
+        ae(filt(None, {'nosy': ['1','2']}, ('+', 'status'),
+            ('-', 'deadline')), ['4', '3'])
 
     def testFilteringMany(self):
         ae, filt = self.filteringSetup()
@@ -896,20 +914,20 @@ class DBTest(MyTestCase):
     def testFilteringRange(self):
         ae, filt = self.filteringSetup()
         # Date ranges
-        ae(filt(None, {'deadline': 'from 2003-02-10 to 2003-02-23'}), ['2','3'])
-        ae(filt(None, {'deadline': '2003-02-10; 2003-02-23'}), ['2','3'])
-        ae(filt(None, {'deadline': '; 2003-02-16'}), ['1'])
+        ae(filt(None, {'deadline': 'from 2003-02-10 to 2003-02-23'}), ['1','3'])
+        ae(filt(None, {'deadline': '2003-02-10; 2003-02-23'}), ['1','3'])
+        ae(filt(None, {'deadline': '; 2003-02-16'}), ['2'])
         # Lets assume people won't invent a time machine, otherwise this test
         # may fail :)
-        ae(filt(None, {'deadline': 'from 2003-02-16'}), ['2', '3', '4'])
-        ae(filt(None, {'deadline': '2003-02-16;'}), ['2', '3', '4'])
+        ae(filt(None, {'deadline': 'from 2003-02-16'}), ['1', '3', '4'])
+        ae(filt(None, {'deadline': '2003-02-16;'}), ['1', '3', '4'])
         # year and month granularity
         ae(filt(None, {'deadline': '2002'}), [])
         ae(filt(None, {'deadline': '2003'}), ['1', '2', '3'])
         ae(filt(None, {'deadline': '2004'}), ['4'])
-        ae(filt(None, {'deadline': '2003-02'}), ['2', '3'])
+        ae(filt(None, {'deadline': '2003-02'}), ['1', '3'])
         ae(filt(None, {'deadline': '2003-03'}), [])
-        ae(filt(None, {'deadline': '2003-02-16'}), ['2'])
+        ae(filt(None, {'deadline': '2003-02-16'}), ['1'])
         ae(filt(None, {'deadline': '2003-02-17'}), [])
         # Interval ranges
         ae(filt(None, {'foo': 'from 0:50 to 2:00'}), ['1'])
@@ -938,32 +956,33 @@ class DBTest(MyTestCase):
         ae(filt(None, {}, ('-','nosy'), (None,None)), ['3', '4', '1', '2'])
 
     def testFilteringDateSort(self):
-        # '1': '2003-01-01.00:00'
-        # '2': '2003-02-16.22:50'
+        # '1': '2003-02-16.22:50'
+        # '2': '2003-01-01.00:00'
         # '3': '2003-02-18'
         # '4': '2004-03-08'
         ae, filt = self.filteringSetup()
         # ascending
-        ae(filt(None, {}, ('+','deadline'), (None,None)), ['1', '2', '3', '4'])
+        ae(filt(None, {}, ('+','deadline'), (None,None)), ['2', '1', '3', '4'])
         # descending
-        ae(filt(None, {}, ('-','deadline'), (None,None)), ['4', '3', '2', '1'])
+        ae(filt(None, {}, ('-','deadline'), (None,None)), ['4', '3', '1', '2'])
 
     def testFilteringDateSortPriorityGroup(self):
-        # '1': '2003-01-01.00:00'  1 => 2
-        # '2': '2003-02-16.22:50'  3 => 1
+        # '1': '2003-02-16.22:50'  1 => 2
+        # '2': '2003-01-01.00:00'  3 => 1
         # '3': '2003-02-18'        2 => 3
         # '4': '2004-03-08'        1 => 2
         ae, filt = self.filteringSetup()
+
         # ascending
         ae(filt(None, {}, ('+','deadline'), ('+','priority')),
-            ['2', '1', '4', '3'])
+            ['2', '1', '3', '4'])
         ae(filt(None, {}, ('-','deadline'), ('+','priority')),
-            ['2', '4', '1', '3'])
+            ['1', '2', '4', '3'])
         # descending
         ae(filt(None, {}, ('+','deadline'), ('-','priority')),
-            ['3', '1', '4', '2'])
+            ['3', '4', '2', '1'])
         ae(filt(None, {}, ('-','deadline'), ('-','priority')),
-            ['3', '4', '1', '2'])
+            ['4', '3', '1', '2'])
 
 # XXX add sorting tests for other types
 # XXX test auditors and reactors
@@ -1191,7 +1210,9 @@ class SchemaTest(MyTestCase):
 
     def init_amod(self):
         self.db = self.module.Database(config, 'admin')
-        a = self.module.Class(self.db, "a", name=String(), fooz=String())
+        a = self.module.Class(self.db, "a", name=String(), newstr=String(),
+            newint=Interval(), newnum=Number(), newbool=Boolean(),
+            newdate=Date())
         a.setkey("name")
         b = self.module.Class(self.db, "b", name=String())
         b.setkey("name")
@@ -1208,18 +1229,22 @@ class SchemaTest(MyTestCase):
         # modify "a" schema
         self.init_amod()
         self.assertEqual(self.db.a.get(aid, 'name'), 'apple')
-        self.assertEqual(self.db.a.get(aid, 'fooz'), None)
+        self.assertEqual(self.db.a.get(aid, 'newstr'), None)
+        self.assertEqual(self.db.a.get(aid, 'newint'), None)
+        self.assertEqual(self.db.a.get(aid, 'newnum'), None)
+        self.assertEqual(self.db.a.get(aid, 'newbool'), None)
+        self.assertEqual(self.db.a.get(aid, 'newdate'), None)
         self.assertEqual(self.db.b.get(aid, 'name'), 'bear')
-        aid2 = self.db.a.create(name='aardvark', fooz='booz')
+        aid2 = self.db.a.create(name='aardvark', newstr='booz')
         self.db.commit(); self.db.close()
 
         # test
         self.init_amod()
         self.assertEqual(self.db.a.get(aid, 'name'), 'apple')
-        self.assertEqual(self.db.a.get(aid, 'fooz'), None)
+        self.assertEqual(self.db.a.get(aid, 'newstr'), None)
         self.assertEqual(self.db.b.get(aid, 'name'), 'bear')
         self.assertEqual(self.db.a.get(aid2, 'name'), 'aardvark')
-        self.assertEqual(self.db.a.get(aid2, 'fooz'), 'booz')
+        self.assertEqual(self.db.a.get(aid2, 'newstr'), 'booz')
 
         # confirm journal's ok
         self.db.getjournal('a', aid)
@@ -1227,8 +1252,8 @@ class SchemaTest(MyTestCase):
 
     def init_amodkey(self):
         self.db = self.module.Database(config, 'admin')
-        a = self.module.Class(self.db, "a", name=String(), fooz=String())
-        a.setkey("fooz")
+        a = self.module.Class(self.db, "a", name=String(), newstr=String())
+        a.setkey("newstr")
         b = self.module.Class(self.db, "b", name=String())
         b.setkey("name")
         self.db.post_init()
@@ -1239,12 +1264,12 @@ class SchemaTest(MyTestCase):
         self.assertEqual(self.db.a.lookup('apple'), aid)
         self.db.commit(); self.db.close()
 
-        # change the key to fooz on a
+        # change the key to newstr on a
         self.init_amodkey()
         self.assertEqual(self.db.a.get(aid, 'name'), 'apple')
-        self.assertEqual(self.db.a.get(aid, 'fooz'), None)
+        self.assertEqual(self.db.a.get(aid, 'newstr'), None)
         self.assertRaises(KeyError, self.db.a.lookup, 'apple')
-        aid2 = self.db.a.create(name='aardvark', fooz='booz')
+        aid2 = self.db.a.create(name='aardvark', newstr='booz')
         self.db.commit(); self.db.close()
 
         # check
@@ -1257,7 +1282,7 @@ class SchemaTest(MyTestCase):
     def init_amodml(self):
         self.db = self.module.Database(config, 'admin')
         a = self.module.Class(self.db, "a", name=String(),
-            fooz=Multilink('a'))
+            newml=Multilink('a'))
         a.setkey('name')
         self.db.post_init()
 
@@ -1269,14 +1294,14 @@ class SchemaTest(MyTestCase):
 
         # add a multilink prop
         self.init_amodml()
-        bid = self.db.a.create(name='bear', fooz=[aid])
-        self.assertEqual(self.db.a.find(fooz=aid), [bid])
+        bid = self.db.a.create(name='bear', newml=[aid])
+        self.assertEqual(self.db.a.find(newml=aid), [bid])
         self.assertEqual(self.db.a.lookup('apple'), aid)
         self.db.commit(); self.db.close()
 
         # check
         self.init_amodml()
-        self.assertEqual(self.db.a.find(fooz=aid), [bid])
+        self.assertEqual(self.db.a.find(newml=aid), [bid])
         self.assertEqual(self.db.a.lookup('apple'), aid)
         self.assertEqual(self.db.a.lookup('bear'), bid)
 
@@ -1288,8 +1313,8 @@ class SchemaTest(MyTestCase):
         # add a multilink prop
         self.init_amodml()
         aid = self.db.a.create(name='apple')
-        bid = self.db.a.create(name='bear', fooz=[aid])
-        self.assertEqual(self.db.a.find(fooz=aid), [bid])
+        bid = self.db.a.create(name='bear', newml=[aid])
+        self.assertEqual(self.db.a.find(newml=aid), [bid])
         self.assertEqual(self.db.a.lookup('apple'), aid)
         self.assertEqual(self.db.a.lookup('bear'), bid)
         self.db.commit(); self.db.close()
