@@ -13,8 +13,12 @@
 import unittest, os, shutil, errno, sys, difflib, cgi, re
 
 from roundup.cgi import client
+from roundup.cgi.exceptions import FormError
+from roundup.cgi.form_parser import FormParser
 from roundup import init, instance, password, hyperdb, date
 
+NEEDS_INSTANCE = 1
+
 class FileUpload:
     def __init__(self, content, filename):
         self.content = content
@@ -68,6 +72,7 @@ class FormTestCase(unittest.TestCase):
         init.install(self.dirname, 'templates/classic')
         init.write_select_db(self.dirname, 'anydbm')
         init.initialise(self.dirname, 'sekrit')
+        
         # check we can load the package
         self.instance = instance.open(self.dirname)
         # and open the database
@@ -85,7 +90,7 @@ class FormTestCase(unittest.TestCase):
 
         # compile the labels re
         classes = '|'.join(self.db.classes.keys())
-        self.FV_SPECIAL = re.compile(client.Client.FV_LABELS%classes,
+        self.FV_SPECIAL = re.compile(FormParser.FV_LABELS%classes,
             re.VERBOSE)
 
     def parseForm(self, form, classname='test', nodeid=None):
@@ -148,21 +153,21 @@ class FormTestCase(unittest.TestCase):
         self.assertEqual(self.parseForm({}), ({('test', None): {}}, []))
 
     def testNothingWithRequired(self):
-        self.assertRaises(ValueError, self.parseForm, {':required': 'string'})
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm, {':required': 'string'})
+        self.assertRaises(FormError, self.parseForm,
             {':required': 'title,status', 'status':'1'}, 'issue')
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm,
             {':required': ['title','status'], 'status':'1'}, 'issue')
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm,
             {':required': 'status', 'status':''}, 'issue')
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm,
             {':required': 'nosy', 'nosy':''}, 'issue')
 
     #
     # Nonexistant edit
     #
     def testEditNonexistant(self):
-        self.assertRaises(IndexError, self.parseForm, {'boolean': ''},
+        self.assertRaises(FormError, self.parseForm, {'boolean': ''},
             'test', '1')
 
     #
@@ -173,7 +178,7 @@ class FormTestCase(unittest.TestCase):
             ({('test', None): {}}, []))
         self.assertEqual(self.parseForm({'string': ' '}),
             ({('test', None): {}}, []))
-        self.assertRaises(ValueError, self.parseForm, {'string': ['', '']})
+        self.assertRaises(FormError, self.parseForm, {'string': ['', '']})
 
     def testSetString(self):
         self.assertEqual(self.parseForm({'string': 'foo'}),
@@ -213,7 +218,7 @@ class FormTestCase(unittest.TestCase):
             ({('test', None): {}}, []))
         self.assertEqual(self.parseForm({'link': ' '}),
             ({('test', None): {}}, []))
-        self.assertRaises(ValueError, self.parseForm, {'link': ['', '']})
+        self.assertRaises(FormError, self.parseForm, {'link': ['', '']})
         self.assertEqual(self.parseForm({'link': '-1'}),
             ({('test', None): {}}, []))
 
@@ -235,8 +240,8 @@ class FormTestCase(unittest.TestCase):
 # XXX This is not the current behaviour - should we enforce this?
 #        self.assertRaises(IndexError, self.parseForm,
 #            {'status': '4'}))
-        self.assertRaises(ValueError, self.parseForm, {'link': 'frozzle'})
-        self.assertRaises(ValueError, self.parseForm, {'status': 'frozzle'},
+        self.assertRaises(FormError, self.parseForm, {'link': 'frozzle'})
+        self.assertRaises(FormError, self.parseForm, {'status': 'frozzle'},
             'issue')
 
     #
@@ -285,11 +290,11 @@ class FormTestCase(unittest.TestCase):
 # XXX This is not the current behaviour - should we enforce this?
 #        self.assertRaises(IndexError, self.parseForm,
 #            {'nosy': '4'}))
-        self.assertRaises(ValueError, self.parseForm, {'nosy': 'frozzle'},
+        self.assertRaises(FormError, self.parseForm, {'nosy': 'frozzle'},
             'issue')
-        self.assertRaises(ValueError, self.parseForm, {'nosy': '1,frozzle'},
+        self.assertRaises(FormError, self.parseForm, {'nosy': '1,frozzle'},
             'issue')
-        self.assertRaises(ValueError, self.parseForm, {'multilink': 'frozzle'})
+        self.assertRaises(FormError, self.parseForm, {'multilink': 'frozzle'})
 
     def testMultilinkAdd(self):
         nodeid = self.db.issue.create(nosy=['1'])
@@ -329,7 +334,7 @@ class FormTestCase(unittest.TestCase):
             'issue', nodeid), ({('issue', nodeid): {'nosy': ['3']}}, []))
 
         # remove one that doesn't exist?
-        self.assertRaises(ValueError, self.parseForm, {':remove:nosy': '4'},
+        self.assertRaises(FormError, self.parseForm, {':remove:nosy': '4'},
             'issue', nodeid)
 
     def testMultilinkRetired(self):
@@ -343,9 +348,9 @@ class FormTestCase(unittest.TestCase):
             ({('issue', nodeid): {'nosy': ['1','2','3']}}, []))
 
     def testAddRemoveNonexistant(self):
-        self.assertRaises(ValueError, self.parseForm, {':remove:foo': '2'},
+        self.assertRaises(FormError, self.parseForm, {':remove:foo': '2'},
             'issue')
-        self.assertRaises(ValueError, self.parseForm, {':add:foo': '2'},
+        self.assertRaises(FormError, self.parseForm, {':add:foo': '2'},
             'issue')
 
     #
@@ -356,9 +361,9 @@ class FormTestCase(unittest.TestCase):
             ({('user', None): {}}, []))
         self.assertEqual(self.parseForm({'password': ''}, 'user'),
             ({('user', None): {}}, []))
-        self.assertRaises(ValueError, self.parseForm, {'password': ['', '']},
+        self.assertRaises(FormError, self.parseForm, {'password': ['', '']},
             'user')
-        self.assertRaises(ValueError, self.parseForm, {'password': 'foo',
+        self.assertRaises(FormError, self.parseForm, {'password': 'foo',
             ':confirm:password': ['', '']}, 'user')
 
     def testSetPassword(self):
@@ -367,9 +372,9 @@ class FormTestCase(unittest.TestCase):
             ({('user', None): {'password': 'foo'}}, []))
 
     def testSetPasswordConfirmBad(self):
-        self.assertRaises(ValueError, self.parseForm, {'password': 'foo'},
+        self.assertRaises(FormError, self.parseForm, {'password': 'foo'},
             'user')
-        self.assertRaises(ValueError, self.parseForm, {'password': 'foo',
+        self.assertRaises(FormError, self.parseForm, {'password': 'foo',
             ':confirm:password': 'bar'}, 'user')
 
     def testEmptyPasswordNotSet(self):
@@ -391,7 +396,7 @@ class FormTestCase(unittest.TestCase):
             ({('test', None): {}}, []))
         self.assertEqual(self.parseForm({'boolean': ' '}),
             ({('test', None): {}}, []))
-        self.assertRaises(ValueError, self.parseForm, {'boolean': ['', '']})
+        self.assertRaises(FormError, self.parseForm, {'boolean': ['', '']})
 
     def testSetBoolean(self):
         self.assertEqual(self.parseForm({'boolean': 'yes'}),
@@ -421,10 +426,10 @@ class FormTestCase(unittest.TestCase):
             ({('test', None): {}}, []))
         self.assertEqual(self.parseForm({'number': ' '}),
             ({('test', None): {}}, []))
-        self.assertRaises(ValueError, self.parseForm, {'number': ['', '']})
+        self.assertRaises(FormError, self.parseForm, {'number': ['', '']})
 
     def testInvalidNumber(self):
-        self.assertRaises(ValueError, self.parseForm, {'number': 'hi, mum!'})
+        self.assertRaises(FormError, self.parseForm, {'number': 'hi, mum!'})
 
     def testSetNumber(self):
         self.assertEqual(self.parseForm({'number': '1'}),
@@ -454,10 +459,10 @@ class FormTestCase(unittest.TestCase):
             ({('test', None): {}}, []))
         self.assertEqual(self.parseForm({'date': ' '}),
             ({('test', None): {}}, []))
-        self.assertRaises(ValueError, self.parseForm, {'date': ['', '']})
+        self.assertRaises(FormError, self.parseForm, {'date': ['', '']})
 
     def testInvalidDate(self):
-        self.assertRaises(ValueError, self.parseForm, {'date': '12'})
+        self.assertRaises(FormError, self.parseForm, {'date': '12'})
 
     def testSetDate(self):
         self.assertEqual(self.parseForm({'date': '2003-01-01'}),
@@ -505,15 +510,15 @@ class FormTestCase(unittest.TestCase):
         )
 
     def testLinkBadDesignator(self):
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm,
             {'test-1@link@link': 'blah'})
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm,
             {'test-1@link@link': 'issue'})
 
     def testLinkNotLink(self):
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm,
             {'test-1@link@boolean': 'issue-1'})
-        self.assertRaises(ValueError, self.parseForm,
+        self.assertRaises(FormError, self.parseForm,
             {'test-1@link@string': 'issue-1'})
 
     def testBackwardsCompat(self):
@@ -528,18 +533,14 @@ class FormTestCase(unittest.TestCase):
             'name': 'foo.txt', 'type': 'text/plain'}},
             [('issue', None, 'files', [('file', '-1')])]))
 
-def suite():
-    l = [
-        unittest.makeSuite(FormTestCase),
-        unittest.makeSuite(MessageTestCase),
-    ]
-    return unittest.TestSuite(l)
+def test_suite():
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(FormTestCase))
+    suite.addTest(unittest.makeSuite(MessageTestCase))
+    return suite
 
-def run():
+if __name__ == '__main__':
     runner = unittest.TextTestRunner()
     unittest.main(testRunner=runner)
 
-if __name__ == '__main__':
-    run()
-
 # vim: set filetype=python ts=4 sw=4 et si
