@@ -372,6 +372,14 @@ class DateTestCase(unittest.TestCase):
         ae('-1y', '1 year ago')
         ae('-2y', '2 years ago')
 
+    def testPyDatetime(self):
+        try:
+            import datetime
+        except:
+            return
+        d = datetime.datetime.now()
+        Date(d)
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(DateTestCase))
