@@ -23,7 +23,7 @@
 import os, unittest, shutil
 
 from roundup.password import Password
-from test_db import setupSchema, MyTestCase, config
+from db_test_base import setupSchema, MyTestCase, config
 
 class PermissionTest(MyTestCase):
     def setUp(self):
@@ -99,8 +99,13 @@ class PermissionTest(MyTestCase):
         self.assertEquals(self.db.security.hasNodePermission('issue',
             issueid, nosy=userid), 1)
 
-def suite():
-    return unittest.makeSuite(PermissionTest)
+def test_suite():
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(PermissionTest))
+    return suite
 
+if __name__ == '__main__':
+    runner = unittest.TextTestRunner()
+    unittest.main(testRunner=runner)
 
 # vim: set filetype=python ts=4 sw=4 et si
